-- UnsheathMove --
-- Created by Vohl - Tarren Mill - EU --

-- Changelog:
-- V1.0 - First release
-- V1.1 - Added options screen and saved variables for default behaviour
-- V1.2 - Added new event for toggle autorun
-- V1.3 - Added New option and saved variable to enable whilst swimming
-- V1.4 - Added a Debug mode 
--      - Move forward and autorun triggers now only fire when NOT mounted 
--      - Toggle from keybind now only functions when NOT mounted  
-- v1.5 - Added an experimental option to keep weapons sheathed
--      - Added a new timer mechanism with option to enable or disable it!
--      - Cleaned up the code executed when moving forwards or autorun
-- v1.5.1.0 - Amended version format 
--			- Added an "on init" variable triggerd by movement to start timer. The timer was firing before the world was loaded causing the sheathstate to display incorrectly
-- v1.5.2.0 - Fixed a bug causing weapons to unsheathe constantly when in a vehicle
-- v1.6.0.0 - Fixed a bug where doing an emote e.g /laugh or typing "lol" in /s would cause the addon to try to unsheath whilst the emote animation was playing. 
--            The timer function will be delayed to allow time for the emote to finish
-- 			- Added a workaround to a game bug. When channeling, weapons will move to the player's back but the sheath state doesnt change which breaks the addon. 
--			  The fix sets a short timer once the channel finishes or cancels and performs a ToggleSheath immediately after
--			- Added a new option to set the refresh rate for the timer if in use
-- v1.6.1.0 - TOC update for patch 7.1
-- v1.6.1.1 - TOC update for patch 7.2.5
-- v1.6.1.2 - Removed on click sounds for options menu to fix sound error when clicking

-- known issue: Addon bugs when casting a spell sometimes. Need to add an event handler to stop the timer when casting and restart when finished.. also... specific handler for hearhtstones, teleports etc



-- initialise the defaults
local frame = CreateFrame("FRAME", "UnsheathMoveFrame");
frame:RegisterEvent("PLAYER_ENTERING_WORLD");
local function eventHandler(self, event, ...)
	if UNSHEATHMOVE_DEFAULT == true then
		UnsheathMoveToggleOnOff = 'Enabled'
		else UnsheathMoveToggleOnOff = 'Disabled'
	end
	if UNSHEATHMOVE_SWIMMING == true then
		UnsheathSwimming = 'Enabled'
	else UnsheathSwimming = 'Disabled'	
	end
	if UNSHEATHMOVE_STAYSHEATHED == true then
		UnsheathStaysheathed = 'Enabled'
		UnsheathStaysheathedToggle = 'Enabled'
		else 
		UnsheathStaysheathed = 'Disabled'
		UnsheathStaysheathedToggle = 'Disabled'
	end
	if UNSHEATHMOVE_TIMER == true then
		UnsheathMoveTimer = 'Enabled'
		else UnsheathMoveTimer = 'Disabled'
	end
	if UNSHEATHMOVE_TIMERVALUE then
		UnsheathMoveTimerValue = UNSHEATHMOVE_TIMERVALUE
		else UnsheathMoveTimerValue = 3
	end
	UnsheathMoveStopTimer()
end	
frame:SetScript("OnEvent", eventHandler);

UnsheathMoveDebug = 'Disabled'
--UnsheathMoveDebug = 'Enabled'


-- set up key bindings
BINDING_HEADER_UNSHEATHMOVE = "UnsheathMove"
BINDING_NAME_UNSHEATHMOVE = "Toggle"

-- Slash Commands
SLASH_UNSHEATHMOVE1 = '/um';
local function handler(param, editbox)
	if param == 'toggle' then
	UnsheathMoveToggle()
  elseif param == 'config' then
	InterfaceOptionsFrame_OpenToCategory("Controls")
	InterfaceOptionsFrame_OpenToCategory("UnsheathMove") -- sometimes the blizz options panel doesnt go to the right page.. this forces it to go to the right page
  elseif param == 'debug' then
		if UnsheathMoveDebug == 'Disabled' then	UnsheathMoveDebug = 'Enabled'
		print('UnsheathMove Debug Mode Enabled')
		else UnsheathMoveDebug = 'Disabled'
		print('UnsheathMove Debug Mode Disabled')
	end	
  else
  print('-----------------------------------------');
  print('UnsheathMove is|cFFFFFF00', UnsheathMoveToggleOnOff, '|r');
  print('Possible options are:');
  print('/um toggle - to toggle on or off');
  print('/um config - to open the options screen');
  print('-----------------------------------------');
end
 end
SlashCmdList["UNSHEATHMOVE"] = handler;

--set up options screen
local UnsheathMoveDefaultText = "Enable the addon on startup"
local UnsheathMoveTimerText = "Enable automatic checking"
local UnsheathMoveSwimmingText = "Enable when swimming (Not recommended)"
local UnsheathMoveStaysheathedText = "Inverse logic - Keep weapons sheathed"
local UnsheathMoveTimerBoxText = "Timer refresh rate in seconds (Default is 3 | Minimum is 1)"
local unsheathmove = CreateFrame("Frame", "unsheathmoveoptions", InterfaceOptionsFramePanelContainer)
unsheathmove:Hide()
unsheathmove.name = "UnsheathMove"
local title = unsheathmove:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
title:SetPoint("TOPLEFT", 16, -16)
title:SetText("UnsheathMove") 
InterfaceOptions_AddCategory(unsheathmove)

--Default Button
local btnUnsheathMoveDefault = CreateFrame("CheckButton", "UnsheathMoveDefaultButton", unsheathmove, "OptionsBaseCheckButtonTemplate")
btnUnsheathMoveDefault:SetPoint("TOPLEFT", 16, -35)
btnUnsheathMoveDefault:SetScript("OnClick", function(frame)
	if frame:GetChecked() then
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
		UNSHEATHMOVE_DEFAULT = true
	else
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF)
		UNSHEATHMOVE_DEFAULT = nil
	end
end)
btnUnsheathMoveDefault:SetScript("OnShow", function(frame)
	frame:SetChecked(UNSHEATHMOVE_DEFAULT)
end)
local btnUnsheathMoveDefaultText = btnUnsheathMoveDefault:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
btnUnsheathMoveDefaultText:SetPoint("LEFT", btnUnsheathMoveDefault, "RIGHT", 0, 1)
btnUnsheathMoveDefaultText:SetText(UnsheathMoveDefaultText)

--Swimming Button
local btnUnsheathMoveSwimming = CreateFrame("CheckButton", "UnsheathMoveSwimmingButton", unsheathmove, "OptionsBaseCheckButtonTemplate")
btnUnsheathMoveSwimming:SetPoint("TOPLEFT", 16, -65)
btnUnsheathMoveSwimming:SetScript("OnClick", function(frame)
	if frame:GetChecked() then
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
		UNSHEATHMOVE_SWIMMING = true
		UnsheathSwimming = 'Enabled'
	else
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF)
		UNSHEATHMOVE_SWIMMING = nil
		UnsheathSwimming = 'Disabled'
	end
end)
btnUnsheathMoveSwimming:SetScript("OnShow", function(frame)
	frame:SetChecked(UNSHEATHMOVE_SWIMMING)
end)
local btnUnsheathMoveSwimmingText = btnUnsheathMoveSwimming:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
btnUnsheathMoveSwimmingText:SetPoint("LEFT", btnUnsheathMoveSwimming, "RIGHT", 0, 1)
btnUnsheathMoveSwimmingText:SetText(UnsheathMoveSwimmingText)

--Stay sheathed button
local btnUnsheathMoveStaysheathed = CreateFrame("CheckButton", "UnsheathMoveStaysheathedButton", unsheathmove, "OptionsBaseCheckButtonTemplate")
btnUnsheathMoveStaysheathed:SetPoint("TOPLEFT", 16, -95)
btnUnsheathMoveStaysheathed:SetScript("OnClick", function(frame)
	if frame:GetChecked() then
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
		UNSHEATHMOVE_STAYSHEATHED = true
		UnsheathStaysheathed = 'Enabled'
		UnsheathStaysheathedToggle = 'Enabled'
	else
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF)
		UNSHEATHMOVE_STAYSHEATHED = nil
		UnsheathStaysheathed = 'Disabled'
		UnsheathStaysheathedToggle = 'Disabled'
	end
end)
btnUnsheathMoveStaysheathed:SetScript("OnShow", function(frame)
	frame:SetChecked(UNSHEATHMOVE_STAYSHEATHED)
end)
local btnUnsheathMoveStaysheathedText = btnUnsheathMoveStaysheathed:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
btnUnsheathMoveStaysheathedText:SetPoint("LEFT", btnUnsheathMoveStaysheathed, "RIGHT", 0, 1)
btnUnsheathMoveStaysheathedText:SetText(UnsheathMoveStaysheathedText)

--Timer Button
local btnUnsheathMoveTimer = CreateFrame("CheckButton", "UnsheathMoveTimerButton", unsheathmove, "OptionsBaseCheckButtonTemplate")
btnUnsheathMoveTimer:SetPoint("TOPLEFT", 16, -125)
btnUnsheathMoveTimer:SetScript("OnClick", function(frame)
	if frame:GetChecked() then
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
		UNSHEATHMOVE_TIMER = true
		UnsheathMoveTimer = 'Enabled'
		timerBox:Show()
	else
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF)
		UNSHEATHMOVE_TIMER = nil
		UnsheathMoveTimer = 'Disabled'
		timerBox:Hide()
	end
end)
btnUnsheathMoveTimer:SetScript("OnShow", function(frame)
	frame:SetChecked(UNSHEATHMOVE_TIMER)
	if UnsheathMoveTimer == 'Enabled' then timerBox:Show()
		else timerBox:Hide()
	end
end)
local btnUnsheathMoveTimerText = btnUnsheathMoveTimer:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
btnUnsheathMoveTimerText:SetPoint("LEFT", btnUnsheathMoveTimer, "RIGHT", 0, 1)
btnUnsheathMoveTimerText:SetText(UnsheathMoveTimerText)

--Timer refresh rate input box
timerBox = CreateFrame("EditBox", "UnsheathMoveTimerBox", unsheathmove, "InputBoxTemplate")
timerBox:SetPoint("TOPLEFT", btnUnsheathMoveTimer, "BOTTOMLEFT", 10, -10)
timerBox:SetAutoFocus(false)
timerBox:SetNumeric(true)
timerBox:SetWidth(30)
timerBox:SetHeight(20)
timerBox:SetMaxLetters(3)
timerBox:SetScript ("OnShow", function(frame)
	frame:SetText(UnsheathMoveTimerValue)
end)

--Refresh rate updates when options form closed
timerBox:SetScript("OnHide", function(frame)
	local n = tonumber(frame:GetText())
	if not n or n == "" and UnsheathMoveTimer == 'Enabled' then 
		frame:SetText(3) 
		print("|cFFFFFF00UnsheathMove Timer enabled but no valid refresh rate found.. Defaulting to|r "..(3)) 
		UNSHEATHMOVE_TIMERVALUE = 3
		UnsheathMoveTimerValue = 3
		return end
	if n < 1 and UnsheathMoveTimer == 'Enabled' then
		frame:SetText(1)
		UNSHEATHMOVE_TIMERVALUE = 1
		UnsheathMoveTimerValue = 1
		print("|cFFFFFF00UnsheathMove Timer minimum refresh rate is 1 second. Defaulting to|r "..(1))
	elseif UnsheathMoveTimer == 'Enabled' then
		UNSHEATHMOVE_TIMERVALUE = n
		UnsheathMoveTimerValue = n
		print("|cFFFFFF00UnsheathMove Timer refresh rate set to|r "..(UNSHEATHMOVE_TIMERVALUE or 3))
	end
	UnsheathMoveStopTimer()
	UnsheathMoveStartTimer()
end)

timerBoxText = timerBox:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
timerBoxText:SetPoint("LEFT", timerBox, "RIGHT", 5, 0)
timerBoxText:SetText(UnsheathMoveTimerBoxText)



-- Debug mode - This prints the current state to the default chat frame everytime the addon triggers
function UnsheathMoveDebugMode()
	if GetSheathState() == 1 then UnsheathMoveDebugSheathState = 'Sheathed'
	end
	if GetSheathState() == 2 then UnsheathMoveDebugSheathState = 'Unsheathed'
	end
	print('UM:|cFFFFFF00', UnsheathMoveToggleOnOff, '|r:: ', 'SS:|cFFFFFF00', UnsheathStaysheathed, '|r::', 'Status:|cFFFFFF00', UnsheathMoveDebugSheathState, '|r:: ', 'Mounted:|cFFFFFF00', IsMounted(), '|r::', 'Timer:|cFFFFFF00', UnsheathMoveTimer, '|r:: ', 'Refresh:|cFFFFFF00', UnsheathMoveTimerValue, '|r:: ')
end

--Debug mode chat frame conditions (stop it from being triigered by the timer)
function UnsheathMoveDebugCheck()
	if UnsheathMoveDebug == 'Enabled' then UnsheathMoveDebugMode()
	end
end

-- Toggle from keybind
function UnsheathMoveToggle()	
	if UnsheathStaysheathed == 'Disabled'
	and not IsMounted() then 
		ToggleSheath()
		UnsheathMoveToggleDefault()
	elseif UnsheathStaysheathed == 'Enabled' then
		ToggleSheath()
		UnsheathMoveStayToggle()
	end
	UnsheathMoveDebugCheck()
end


-- Toggles default and checks against actual sheathstate to set variable
function UnsheathMoveToggleDefault()
	if UnsheathMoveToggleOnOff == 'Disabled' and GetSheathState() == 1
		then UnsheathMoveToggleOnOff = 'Enabled'
		elseif UnsheathMoveToggleOnOff == 'Enabled' and GetSheathState() == 1
		then UnsheathMoveToggleOnOff = 'Disabled'
		else
		UnsheathMoveToggleOnOff = 'Disabled'
	end
end

--Toggles if Stay sheathed is enabled
function UnsheathMoveStayToggle()
	if UnsheathStaysheathedToggle == 'Enabled' and GetSheathState() == 1
		then UnsheathStaysheathedToggle = 'Disabled'
		elseif UnsheathStaysheathedToggle == 'Disabled' and GetSheathState() == 1
		then UnsheathStaysheathedToggle = 'Enabled'
		else UnsheathStaysheathedToggle = 'Enabled'
	end
end

--Function to keep weapons unsheathed
function UnsheathUnsheathFunc()
	if GetSheathState() == 1 
	and (not IsSwimming() or (UnsheathSwimming == 'Enabled' and IsSwimming()))
	and not IsMounted()
	and UnitInVehicle("Player") == false
	and UnsheathMoveToggleOnOff == 'Enabled'
	and UnsheathStaysheathed == 'Disabled'
	and (UnitCastingInfo("Player") == nil or UnitChannelInfo("Player") == nil)
	then ToggleSheath()
	UnsheathMoveToggleOnOff = 'Enabled'
	end
end

--Function to keep weapons sheathed
function UnsheathStaysheathedFunc()
	if GetSheathState() == 2 
	and UnsheathStaysheathed == 'Enabled'
	and UnsheathStaysheathedToggle == 'Enabled'
	and UnitAffectingCombat("Player") == false
	then ToggleSheath()
	UnsheathMoveToggleOnOff = 'Enabled'
	end
end

--unsheath if enabled on moving forward
hooksecurefunc("MoveForwardStart", function()
	UnsheathUnsheathFunc()
	UnsheathStaysheathedFunc()
	UnsheathMoveDebugCheck()
	UnsheathMoveTimerInit = 'Init'
	UnsheathMoveStartTimer()
end)

--unsheath if enabled and autorun is toggled
hooksecurefunc("ToggleAutoRun", function()
	UnsheathUnsheathFunc()
	UnsheathStaysheathedFunc()
	UnsheathMoveDebugCheck()
	UnsheathMoveTimerInit = 'Init'
	UnsheathMoveStartTimer()
end)

--Timers
local UnsheathTimer = 0
local function unsheathMoveTimerStart(self,elapsed)
	if UnsheathMoveTimer == 'Enabled' and UnsheathMoveTimerInit == 'Init' then
		UnsheathTimer = UnsheathTimer + elapsed
			if UnsheathTimer >= (tonumber(UnsheathMoveTimerValue)) then
			UnsheathUnsheathFunc()
			UnsheathStaysheathedFunc()
			UnsheathMoveDebugCheck()
			UnsheathTimer = 0
		end
	end
end

--One hit timer when a channel cancels or finishes. When it expires it then restarts the unsheathing timer and performs a "toggle sheath". This is a workaround for a bug in game see changelog for details 
UnsheathChannelTimer = 0
local function unsheathMoveChannelEnd(self,elapsed)
	UnsheathChannelTimer = UnsheathChannelTimer + elapsed
	--print("channel finished.. delaying timer")
	if UnsheathChannelTimer >= 1 then
		UnsheathMoveStartTimer()
		UnsheathChannelTimer = 0
		UnsheathMoveChannelEndTimerStop()
	end
	if UnsheathStaysheathedToggle == 'Disabled' and UnsheathMoveToggleOnOff == 'Enabled' then
		ToggleSheath()
	end
end


local unsheathTimerFrame = CreateFrame("frame")
local unsheathMoveChannelEndFrame = CreateFrame("frame")

--Start Timer
function UnsheathMoveStartTimer()	
	unsheathTimerFrame:SetScript("OnUpdate", unsheathMoveTimerStart)
end

--Stop Timer
function UnsheathMoveStopTimer()
	unsheathTimerFrame:SetScript("OnUpdate", nil)
end

--Trigger for the channel end 1 hit timer
function UnsheathChannelToggle()
	--print("player finished channeling")
	unsheathMoveChannelEndFrame:SetScript("OnUpdate", unsheathMoveChannelEnd)
 end

 --Stop timer for channel ending
function UnsheathMoveChannelEndTimerStop()
	unsheathMoveChannelEndFrame:SetScript("OnUpdate", nil)
end
 
--Event handler for emotes. Adds a delay to the refresh timer to prevent the addon unsheathing during an emote 
local unsheathtimerdelay = CreateFrame("FRAME", "");
unsheathtimerdelay:RegisterEvent("CHAT_MSG_SAY");
unsheathtimerdelay:RegisterEvent("CHAT_MSG_YELL");
unsheathtimerdelay:RegisterEvent("CHAT_MSG_TEXT_EMOTE");
local function timerDelay(self, event, ...)
	if GetUnitName("player") == select(2,...) and event == "CHAT_MSG_TEXT_EMOTE" or event == "CHAT_MSG_SAY" or event == "CHAT_MSG_YELL" then
		UnsheathTimer = -1
	end
end
unsheathtimerdelay:SetScript("OnEvent", timerDelay);

--Event handler for channels. When a channeled spell is detected it stops the timer to prevent unsheating occuring during the channel. When the channel then finishes or cancels the timer will restart after a 3 second delay to prevent any UI glitches
local unsheathchanneling = CreateFrame("FRAME", "");
unsheathchanneling:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
unsheathchanneling:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
unsheathchanneling:RegisterEvent("UNIT_SPELLCAST_CHANNEL_SUCCEEDED");
local function unsheathChannel(self, event, ...)
	if event == "UNIT_SPELLCAST_CHANNEL_START" then UnsheathMoveStopTimer()
	elseif event == "UNIT_SPELLCAST_CHANNEL_STOP" or event == "UNIT_SPELLCAST_CHANNEL_SUCCEEDED" then
		UnsheathChannelToggle()
	end
end
unsheathchanneling:SetScript("OnEvent", unsheathChannel);
